@@ -1,19 +1,19 @@
 /*--------------------------------*-C-*---------------------------------*
  * File:	logging.c
  *----------------------------------------------------------------------*
- * $Id: logging.c,v 1.25 2001/04/02 17:04:55 gcw Exp $
+ * $Id: logging.c,v 1.38 2002/12/16 00:29:30 gcw Exp $
  *
  * All portions of code are copyright by their respective author/s.
- * Copyright (C) 1992      John Bovey <jdb@ukc.ac.uk>
+ * Copyright (c) 1992      John Bovey <jdb@ukc.ac.uk>
  *				- original version
- * Copyright (C) 1993      lipka
- * Copyright (C) 1993      Brian Stempien <stempien@cs.wmich.edu>
- * Copyright (C) 1995      Raul Garcia Garcia <rgg@tid.es>
- * Copyright (C) 1995      Piet W. Plomp <piet@idefix.icce.rug.nl>
- * Copyright (C) 1997      Raul Garcia Garcia <rgg@tid.es>
- * Copyright (C) 1998-2001 Geoff Wing <gcw@pobox.com>
+ * Copyright (c) 1993      lipka
+ * Copyright (c) 1993      Brian Stempien <stempien@cs.wmich.edu>
+ * Copyright (c) 1995      Raul Garcia Garcia <rgg@tid.es>
+ * Copyright (c) 1995      Piet W. Plomp <piet@idefix.icce.rug.nl>
+ * Copyright (c) 1997      Raul Garcia Garcia <rgg@tid.es>
+ * Copyright (c) 1998-2001 Geoff Wing <gcw@pobox.com>
  * 				- extensive modifications
- * Copyright (C) 1999      D J Hawkey Jr <hawkeyd@visi.com>
+ * Copyright (c) 1999      D J Hawkey Jr <hawkeyd@visi.com>
  *				- lastlog support
  *
  * This program is free software; you can redistribute it and/or modify
@@ -44,29 +44,293 @@
 #include "logging.h"
 #ifdef UTMP_SUPPORT
 
-/*
- * HAVE_SETUTENT corresponds to SYSV-style utmp support.
- * Without it corresponds to using BSD utmp support.
- * Exception: QNX has setutent(), yet is a mix of BSD and SYSV.
- * SYSV-style utmp support is further divided in normal utmp support
- * and utmpx support (Solaris 2.x) by RXVT_UTMP_AS_UTMPX
- */
-
 /*
  * BSD style utmp entry
  *      ut_line, ut_name, ut_host, ut_time
- * SYSV style utmp entry
+ * SYSV style utmp (and utmpx) entry
  *      ut_user, ut_id, ut_line, ut_pid, ut_type, ut_exit, ut_time
  */
 
-/*
- * update wtmp entries
- */
 /* ------------------------------------------------------------------------- */
-#if defined(WTMP_SUPPORT) && !defined(RXVT_UTMP_AS_UTMPX)
+
+/*
+ * make and write utmp and wtmp entries
+ */
+/* EXTPROTO */
+void
+rxvt_makeutent(rxvt_t *r, const char *pty, const char *hostname)
+{
+#ifdef HAVE_STRUCT_UTMP
+    struct utmp    *ut = &(r->h->ut);
+#endif
+#ifdef HAVE_STRUCT_UTMPX
+    struct utmpx   *utx = &(r->h->utx);
+#endif
+#ifdef HAVE_UTMP_PID
+    int             i;
+#endif
+    char            ut_id[5];
+    struct passwd  *pwent = getpwuid(getuid());
+
+    if (!STRNCMP(pty, "/dev/", 5))
+	pty += 5;		/* skip /dev/ prefix */
+
+    if (!STRNCMP(pty, "pty", 3) || !STRNCMP(pty, "tty", 3)) {
+	STRNCPY(ut_id, (pty + 3), sizeof(ut_id));
+    }
+#ifdef HAVE_UTMP_PID
+    else if (sscanf(pty, "pts/%d", &i) == 1)
+	sprintf(ut_id, "vt%02x", (i & 0xff));	/* sysv naming */
+#endif
+    else if (STRNCMP(pty, "pty", 3) && STRNCMP(pty, "tty", 3)) {
+	rxvt_print_error("can't parse tty name \"%s\"", pty);
+	return;
+    }
+
+#ifdef HAVE_STRUCT_UTMP
+    MEMSET(ut, 0, sizeof(struct utmp));
+# ifdef HAVE_UTMP_PID
+    setutent();
+    STRNCPY(ut->ut_id, ut_id, sizeof(ut->ut_id));
+    ut->ut_type = DEAD_PROCESS;
+    getutid(ut);		/* position to entry in utmp file */
+    STRNCPY(r->h->ut_id, ut_id, sizeof(r->h->ut_id));
+# endif
+#endif
+
+#ifdef HAVE_STRUCT_UTMPX
+    MEMSET(utx, 0, sizeof(struct utmpx));
+    setutxent();
+    STRNCPY(utx->ut_id, ut_id, sizeof(utx->ut_id));
+    utx->ut_type = DEAD_PROCESS;
+    getutxid(utx);		/* position to entry in utmp file */
+    STRNCPY(r->h->ut_id, ut_id, sizeof(r->h->ut_id));
+#endif
+
+#ifdef HAVE_STRUCT_UTMP
+    STRNCPY(ut->ut_line, pty, sizeof(ut->ut_line));
+    ut->ut_time = time(NULL);
+# ifdef HAVE_UTMP_PID
+    STRNCPY(ut->ut_user, (pwent && pwent->pw_name) ? pwent->pw_name : "?",
+	    sizeof(ut->ut_user));
+    STRNCPY(ut->ut_id, ut_id, sizeof(ut->ut_id));
+    ut->ut_time = time(NULL);
+    ut->ut_pid = r->h->cmd_pid;
+#  ifdef HAVE_UTMP_HOST
+    STRNCPY(ut->ut_host, hostname, sizeof(ut->ut_host));
+#  endif
+    ut->ut_type = USER_PROCESS;
+    pututline(ut);
+    endutent();			/* close the file */
+    r->h->utmp_pos = -1;
+# else
+    STRNCPY(ut->ut_name, (pwent && pwent->pw_name) ? pwent->pw_name : "?",
+	    sizeof(ut->ut_name));
+#  ifdef HAVE_UTMP_HOST
+    STRNCPY(ut->ut_host, hostname, sizeof(ut->ut_host));
+#  endif
+# endif
+#endif
+
+#ifdef HAVE_STRUCT_UTMPX
+    STRNCPY(utx->ut_line, pty, sizeof(utx->ut_line));
+    STRNCPY(utx->ut_user, (pwent && pwent->pw_name) ? pwent->pw_name : "?",
+	    sizeof(utx->ut_user));
+    STRNCPY(utx->ut_id, ut_id, sizeof(utx->ut_id));
+    utx->ut_session = getsid(0);
+    utx->ut_tv.tv_sec = time(NULL);
+    utx->ut_tv.tv_usec = 0;
+    utx->ut_pid = r->h->cmd_pid;
+# ifdef HAVE_UTMPX_HOST
+    STRNCPY(utx->ut_host, hostname, sizeof(utx->ut_host));
+#  if 0
+    {
+	char           *colon;
+
+	if ((colon = STRRCHR(ut->ut_host, ':')) != NULL)
+	    *colon = '\0';
+    }
+#  endif
+# endif
+    utx->ut_type = USER_PROCESS;
+    pututxline(utx);
+    endutxent();		/* close the file */
+    r->h->utmp_pos = -1;
+#endif
+
+#if defined(HAVE_STRUCT_UTMP) && !defined(HAVE_UTMP_PID)
+    {
+	int             i;
+# ifdef HAVE_TTYSLOT
+	i = ttyslot();
+	if (rxvt_write_bsd_utmp(i, ut))
+	    r->h->utmp_pos = i;
+# else
+	FILE           *fd0;
+
+	if ((fd0 = fopen(TTYTAB_FILENAME, "r")) != NULL) {
+	    char            buf[256], name[256];
+
+	    buf[sizeof(buf) - 1] = '\0';
+	    for (i = 1; (fgets(buf, sizeof(buf) - 1, fd0) != NULL);) {
+		if (*buf == '#' || sscanf(buf, "%s", name) != 1)
+		    continue;
+		if (!STRCMP(ut->ut_line, name)) {
+		    if (!rxvt_write_bsd_utmp(i, ut))
+			i = 0;
+		    r->h->utmp_pos = i;
+		    fclose(fd0);
+		    break;
+		}
+		i++;
+	    }
+	    fclose(fd0);
+	}
+# endif
+    }
+#endif
+
+#ifdef WTMP_SUPPORT
+# ifdef WTMP_ONLY_ON_LOGIN
+    if (r->Options & Opt_loginShell)
+# endif
+    {
+# ifdef HAVE_STRUCT_UTMP
+#  ifdef HAVE_UPDWTMP
+	updwtmp(RXVT_WTMP_FILE, ut);
+#  else
+	rxvt_update_wtmp(RXVT_WTMP_FILE, ut);
+#  endif
+# endif
+# ifdef HAVE_STRUCT_UTMPX
+	updwtmpx(RXVT_WTMPX_FILE, utx);
+# endif
+    }
+#endif
+#if defined(LASTLOG_SUPPORT) && defined(RXVT_LASTLOG_FILE)
+    if (r->Options & Opt_loginShell)
+	rxvt_update_lastlog(RXVT_LASTLOG_FILE, pty, hostname);
+#endif
+}
+
+/* ------------------------------------------------------------------------- */
+/*
+ * remove utmp and wtmp entries
+ */
+/* EXTPROTO */
+void
+rxvt_cleanutent(rxvt_t *r)
+{
+#ifdef HAVE_STRUCT_UTMP
+    struct utmp    *tmput, *ut = &(r->h->ut);
+#endif
+#ifdef HAVE_STRUCT_UTMPX
+    struct utmpx   *tmputx, *utx = &(r->h->utx);
+#endif
+
+#ifdef HAVE_STRUCT_UTMP
+# ifdef HAVE_UTMP_PID
+    MEMSET(ut, 0, sizeof(struct utmp));
+    setutent();
+    STRNCPY(ut->ut_id, r->h->ut_id, sizeof(ut->ut_id));
+    ut->ut_type = USER_PROCESS;
+    if ((tmput = getutid(ut)))	/* position to entry in utmp file */
+	ut = tmput;
+    ut->ut_type = DEAD_PROCESS;
+# else 
+    MEMSET(ut->ut_name, 0, sizeof(ut->ut_name));
+#  ifdef HAVE_UTMP_HOST
+    MEMSET(ut->ut_host, 0, sizeof(ut->ut_host));
+#  endif
+# endif
+    ut->ut_time = time(NULL);
+#endif
+
+#ifdef HAVE_STRUCT_UTMPX
+    MEMSET(utx, 0, sizeof(struct utmpx));
+    setutxent();
+    STRNCPY(utx->ut_id, r->h->ut_id, sizeof(utx->ut_id));
+    utx->ut_type = USER_PROCESS;
+    if ((tmputx = getutxid(utx)))	/* position to entry in utmp file */
+	utx = tmputx;
+    utx->ut_type = DEAD_PROCESS;
+    utx->ut_session = getsid(0);
+    utx->ut_tv.tv_sec = time(NULL);
+    utx->ut_tv.tv_usec = 0;
+#endif
+
+    /*
+     * Write ending wtmp entry
+     */
+#ifdef WTMP_SUPPORT
+# ifdef WTMP_ONLY_ON_LOGIN
+    if (r->Options & Opt_loginShell)
+# endif
+    {
+# ifdef HAVE_STRUCT_UTMP
+#  ifdef HAVE_UPDWTMP
+	updwtmp(RXVT_WTMP_FILE, ut);
+#  else
+	rxvt_update_wtmp(RXVT_WTMP_FILE, ut);
+#  endif
+# endif
+# ifdef HAVE_STRUCT_UTMPX
+	updwtmpx(RXVT_WTMPX_FILE, utx);
+# endif
+    }
+#endif
+
+    /*
+     * Write utmp entry
+     */
+#ifdef HAVE_STRUCT_UTMP
+# ifdef HAVE_UTMP_PID
+    if (ut->ut_pid == r->h->cmd_pid)
+	pututline(ut);
+    endutent();
+# else
+    if (r->h->utmp_pos > 0) {
+	MEMSET(ut, 0, sizeof(struct utmp));
+	rxvt_write_bsd_utmp(r->h->utmp_pos, ut);
+    }
+# endif
+#endif
+#ifdef HAVE_STRUCT_UTMPX
+    if (utx->ut_pid == r->h->cmd_pid)
+	pututxline(utx);
+    endutxent();
+#endif
+}
+
+/* ------------------------------------------------------------------------- */
+/*
+ * Write a BSD style utmp entry
+ */
+#ifdef HAVE_UTMP_H
+/* INTPROTO */
+int
+rxvt_write_bsd_utmp(int utmp_pos, struct utmp *wu)
+{
+    int             fd;
+
+    if (utmp_pos <= 0 || (fd = open(RXVT_UTMP_FILE, O_WRONLY)) == -1)
+	return 0;
+
+    if (lseek(fd, (off_t) (utmp_pos * sizeof(struct utmp)), SEEK_SET) != -1)
+	write(fd, wu, sizeof(struct utmp));
+    close(fd);
+    return 1;
+}
+#endif
+
+/* ------------------------------------------------------------------------- */
+/*
+ * Update a BSD style wtmp entry
+ */
+#if defined(WTMP_SUPPORT) && !defined(HAVE_UPDWTMP)
 /* INTPROTO */
 void
-rxvt_update_wtmp(const char *fname, const UTMP *putmp)
+rxvt_update_wtmp(const char *fname, const struct utmp *putmp)
 {
     int             fd, gotlock, retry;
     struct flock    lck;	/* fcntl locking scheme */
@@ -92,7 +356,7 @@ rxvt_update_wtmp(const char *fname, const UTMP *putmp)
 	return;
     }
     if (fstat(fd, &sbuf) == 0)
-	if (write(fd, putmp, sizeof(UTMP)) != sizeof(UTMP))
+	if (write(fd, putmp, sizeof(struct utmp)) != sizeof(struct utmp))
 	    ftruncate(fd, sbuf.st_size);	/* remove bad writes */
 
     lck.l_type = F_UNLCK;	/* unlocking the file */
@@ -100,39 +364,35 @@ rxvt_update_wtmp(const char *fname, const UTMP *putmp)
     close(fd);
 }
 #endif
+
 /* ------------------------------------------------------------------------- */
-
-#ifndef USE_SYSV_UTMP
-/* INTPROTO */
-int
-rxvt_write_bsd_utmp(int utmp_pos, UTMP *wu)
-{
-    int             fd;
-
-    if (utmp_pos <= 0 || (fd = open(RXVT_REAL_UTMP_FILE, O_WRONLY)) == -1)
-	return 0;
-
-    lseek(fd, (off_t) (utmp_pos * sizeof(UTMP)), SEEK_SET);
-    write(fd, wu, sizeof(UTMP));
-    close(fd);
-    return 1;
-}
-#endif
-/* ------------------------------------------------------------------------- */
-
 #ifdef LASTLOG_SUPPORT
 /* INTPROTO */
 void
 rxvt_update_lastlog(const char *fname, const char *pty, const char *host)
 {
+# ifdef HAVE_STRUCT_LASTLOGX
+    struct lastlogx llx;
+# endif
 # ifdef HAVE_STRUCT_LASTLOG
     int             fd;
-    struct passwd  *pwent;
     struct lastlog  ll;
 #  ifdef LASTLOG_IS_DIR
     char            lastlogfile[256];
 #  endif
+    struct passwd  *pwent;
+# endif
 
+# ifdef HAVE_STRUCT_LASTLOGX
+    MEMSET(&llx, 0, sizeof(llx));
+    llx.ll_tv.tv_sec = time(NULL);
+    llx.ll_tv.tv_usec = 0;
+    STRNCPY(llx.ll_line, pty, sizeof(llx.ll_line));
+    STRNCPY(llx.ll_host, host, sizeof(llx.ll_host));
+    updlastlogx(RXVT_LASTLOGX_FILE, getuid(), &llx);
+# endif
+
+# ifdef HAVE_STRUCT_LASTLOG
     pwent = getpwuid(getuid());
     if (!pwent) {
 	rxvt_print_error("no entry in password file");
@@ -154,8 +414,9 @@ rxvt_update_lastlog(const char *fname, const char *pty, const char *host)
     }
 #  else
     if ((fd = open(fname, O_RDWR)) != -1) {
-	lseek(fd, (off_t) ((long)pwent->pw_uid * sizeof(ll)), SEEK_SET);
-	write(fd, &ll, sizeof(ll));
+	if (lseek(fd, (off_t) ((long)pwent->pw_uid * sizeof(ll)),
+		  SEEK_SET) != -1)
+	    write(fd, &ll, sizeof(ll));
 	close(fd);
     }
 #  endif			/* LASTLOG_IS_DIR */
@@ -164,242 +425,4 @@ rxvt_update_lastlog(const char *fname, const char *pty, const char *host)
 #endif				/* LASTLOG_SUPPORT */
 /* ------------------------------------------------------------------------- */
 
-/*
- * make a utmp entry
- */
-/* EXTPROTO */
-void
-rxvt_makeutent(rxvt_t *r, const char *pty, const char *hostname)
-#ifndef USE_SYSV_UTMP
-/* ------------------------------ BSD ------------------------------  */
-{
-    FILE           *fd0;
-    UTMP           *ut = &(r->h->ut);
-    struct passwd  *pwent = getpwuid(getuid());
-#ifdef __QNX__
-    UTMP            u2;
-    struct utsname  un;
-#endif
-    char            buf[256], name[256];
-
-/* BSD naming is of the form /dev/tty?? or /dev/pty?? */
-    MEMSET(ut, 0, sizeof(UTMP));
-
-    if (!STRNCMP(pty, "/dev/", 5))
-	pty += 5;		/* skip /dev/ prefix */
-    if (STRNCMP(pty, "pty", 3) && STRNCMP(pty, "tty", 3)) {
-	rxvt_print_error("can't parse tty name \"%s\"", pty);
-	return;
-    }
-#ifdef __QNX__
-    uname(&un);
-    STRNCPY(ut->id, un.nodename, sizeof(ut->ut_line));
-    ut->ut_pid = getpid();
-    ut->ut_type = USER_PROCESS;
-#endif
-    STRNCPY(ut->ut_line, pty, sizeof(ut->ut_line));
-    STRNCPY(ut->ut_name, (pwent && pwent->pw_name) ? pwent->pw_name : "?",
-	    sizeof(ut->ut_name));
-    STRNCPY(ut->ut_host, hostname, sizeof(ut->ut_host));
-    ut->ut_time = time(NULL);
-
-    buf[sizeof(buf) - 1] = '\0';
-#ifdef __QNX__
-    for (;;) {
-	utmp_pos = fseek(fd0, 0L, 1);
-	if (fread(&u2, sizeof(UTMP), 1, fd0) != 1)
-	    break;
-	if ((u2.ut_type == LOGIN_PROCESS || u2.ut_type == USER_PROCESS)
-	    && (STRNCMP(u2.ut_line, pty, sizeof(u2.ut_line)) == 0))
-	    break;
-    }
-#else
-    if ((fd0 = fopen(TTYTAB_FILENAME, "r")) != NULL) {
-	int             i;
-
-	for (i = 1; (fgets(buf, sizeof(buf) - 1, fd0) != NULL);) {
-	    if (*buf == '#' || sscanf(buf, "%s", name) != 1)
-		continue;
-	    if (!STRCMP(ut->ut_line, name)) {
-		r->h->utmp_pos = i;
-		fclose(fd0);
-		break;
-	    }
-	    i++;
-	}
-	fclose(fd0);
-    }
-#endif
-    if (!rxvt_write_bsd_utmp(r->h->utmp_pos, ut))
-	r->h->utmp_pos = 0;
-#ifdef WTMP_SUPPORT
-# ifdef WTMP_ONLY_ON_LOGIN
-    if (r->Options & Opt_loginShell)
-# endif
-	update_wtmp(RXVT_REAL_WTMP_FILE, ut);
-#endif
-#if defined(LASTLOG_SUPPORT) && defined(RXVT_LASTLOG_FILE)
-    if (r->Options & Opt_loginShell)
-	rxvt_update_lastlog(RXVT_LASTLOG_FILE, pty, hostname);
-#endif
-}
-#else
-/* ------------------------------ SYSV ------------------------------  */
-{
-    int             i;
-    char           *colon;
-    UTMP           *ut = &(r->h->ut);
-    struct passwd  *pwent = getpwuid(getuid());
-
-    MEMSET(ut, 0, sizeof(UTMP));
-
-    if (!STRNCMP(pty, "/dev/", 5))
-	pty += 5;		/* skip /dev/ prefix */
-    if (!STRNCMP(pty, "pty", 3) || !STRNCMP(pty, "tty", 3))
-	STRNCPY(ut->ut_id, (pty + 3), sizeof(ut->ut_id));
-    else if (sscanf(pty, "pts/%d", &i) == 1)
-	sprintf(ut->ut_id, "vt%02x", (i & 0xff));	/* sysv naming */
-    else {
-	rxvt_print_error("can't parse tty name \"%s\"", pty);
-	return;
-    }
-
-#if 0
-    /* XXX: most likely unnecessary.  could be harmful */
-    utmpname(RXVT_REAL_UTMP_FILE);
-#endif
-    STRNCPY(r->h->ut_id, ut->ut_id, sizeof(r->h->ut_id));
-
-    setutent();			/* XXX: should be unnecessaray */
-
-    ut->ut_type = DEAD_PROCESS;
-    getutid(ut);		/* position to entry in utmp file */
-
-/* set up the new entry */
-    ut->ut_type = USER_PROCESS;
-#ifndef linux
-    ut->ut_exit.e_exit = 2;
-#endif
-    STRNCPY(ut->ut_user, (pwent && pwent->pw_name) ? pwent->pw_name : "?",
-	    sizeof(ut->ut_user));
-/* ut_name is normally the same as ut_user, but .... */
-    STRNCPY(ut->ut_name, (pwent && pwent->pw_name) ? pwent->pw_name : "?",
-	    sizeof(ut->ut_name));
-    STRNCPY(ut->ut_id, r->h->ut_id, sizeof(ut->ut_id));
-    STRNCPY(ut->ut_line, pty, sizeof(ut->ut_line));
-
-#if (defined(HAVE_UTMP_HOST) && ! defined(RXVT_UTMP_AS_UTMPX)) || (defined(HAVE_UTMPX_HOST) && defined(RXVT_UTMP_AS_UTMPX))
-    STRNCPY(ut->ut_host, hostname, sizeof(ut->ut_host));
-# ifndef linux
-    if ((colon = STRRCHR(ut->ut_host, ':')) != NULL)
-	*colon = '\0';
-# endif
-#endif
-
-    ut->ut_pid = getpid();
-
-#ifdef RXVT_UTMP_AS_UTMPX
-    ut->ut_session = getsid(0);
-    ut->ut_tv.tv_sec = time(NULL);
-    ut->ut_tv.tv_usec = 0;
-#else
-    ut->ut_time = time(NULL);
-#endif				/* HAVE_UTMPX_H */
-    pututline(ut);
-    r->h->utmp_pos = 1;
-
-#ifdef WTMP_SUPPORT
-# ifdef WTMP_ONLY_ON_LOGIN
-    if (r->Options & Opt_loginShell)
-# endif
-	update_wtmp(RXVT_REAL_WTMP_FILE, ut);
-#endif
-#if defined(LASTLOG_SUPPORT) && defined(RXVT_LASTLOG_FILE)
-    if (r->Options & Opt_loginShell)
-	rxvt_update_lastlog(RXVT_LASTLOG_FILE, pty, hostname);
-#endif
-    endutent();			/* close the file */
-}
-#endif				/* !USE_SYSV_UTMP */
-
-/* ------------------------------------------------------------------------- */
-/*
- * remove a utmp entry
- */
-/* EXTPROTO */
-void
-rxvt_cleanutent(rxvt_t *r)
-#ifndef USE_SYSV_UTMP
-/* ------------------------------ BSD ------------------------------  */
-{
-    UTMP           *ut = &(r->h->ut);
-#ifdef __QNX__
-    UTMP            u2;
-
-    MEMCPY(u2, ut, sizeof(UTMP));
-#endif
-#ifdef WTMP_SUPPORT
-# ifdef WTMP_ONLY_ON_LOGIN
-    if (r->Options & Opt_loginShell)
-# endif
-    {
-	MEMSET(ut->ut_name, 0, sizeof(ut->ut_name));
-	MEMSET(ut->ut_host, 0, sizeof(ut->ut_host));
-	ut->ut_time = time(NULL);
-	update_wtmp(RXVT_REAL_WTMP_FILE, ut);
-    }
-#endif
-    if (!r->h->utmp_pos)
-	return;
-#ifdef __QNX__
-    u2.ut_type = DEAD_PROCESS;
-    rxvt_write_bsd_utmp(r->h->utmp_pos, &u2);
-#else
-    MEMSET(ut, 0, sizeof(UTMP));
-    rxvt_write_bsd_utmp(r->h->utmp_pos, ut);
-#endif
-}
-#else				/* USE_SYSV_UTMP */
-/* ------------------------------ SYSV ------------------------------  */
-{
-    UTMP           *putmp, *ut = &(r->h->ut);
-
-    if (!r->h->utmp_pos)
-	return;
-
-#if 0
-    /* XXX: most likely unnecessary.  could be harmful */
-    utmpname(RXVT_REAL_UTMP_FILE);
-#endif
-    MEMSET(ut, 0, sizeof(UTMP));
-    STRNCPY(ut->ut_id, r->h->ut_id, sizeof(ut->ut_id));
-    ut->ut_type = USER_PROCESS;
-
-    setutent();			/* XXX: should be unnecessaray */
-
-    putmp = getutid(ut);
-    if (!putmp || putmp->ut_pid != getpid())
-	return;
-
-    putmp->ut_type = DEAD_PROCESS;
-
-#ifdef RXVT_UTMP_AS_UTMPX
-    putmp->ut_session = getsid(0);
-    putmp->ut_tv.tv_sec = time(NULL);
-    putmp->ut_tv.tv_usec = 0;
-#else				/* HAVE_UTMPX_H */
-    putmp->ut_time = time(NULL);
-#endif				/* HAVE_UTMPX_H */
-    pututline(putmp);
-
-#ifdef WTMP_SUPPORT
-# ifdef WTMP_ONLY_ON_LOGIN
-    if (r->Options & Opt_loginShell)
-# endif
-	update_wtmp(RXVT_REAL_WTMP_FILE, putmp);
-#endif
-
-    endutent();
-}
-#endif				/* !USE_SYSV_UTMP */
 #endif				/* UTMP_SUPPORT */
