@@ -1,9 +1,11 @@
 /* Include prototypes for all files */
 /*
- * $Id: protos.h,v 1.10 2000/01/14 01:26:24 mason Exp $
+ * $Id: protos.h,v 1.14 2001/04/02 07:32:35 gcw Exp $
  */
 #include "command.extpro"
 
+#include "defaultfont.extpro"
+
 #ifdef RXVT_GRAPHICS
 # include "graphics.extpro"
 #endif
@@ -29,13 +31,24 @@
 # include "netdisp.extpro"
 #endif
 
-#ifndef NO_STRINGS
+#include "ptytty.extpro"
+
+#if !defined(NO_STRINGS) && !defined(HAVE_STRING_H)
 # include "strings.extpro"
 #endif
 
 #include "screen.extpro"
 
 #include "scrollbar.extpro"
+#ifdef RXVT_SCROLLBAR
+# include "scrollbar-rxvt.extpro"
+#endif
+#ifdef NEXT_SCROLLBAR
+# include "scrollbar-next.extpro"
+#endif
+#ifdef XTERM_SCROLLBAR
+# include "scrollbar-xterm.extpro"
+#endif
 
 #include "xdefaults.extpro"
 
